/*
 * Copyright 2016 - 2022 Gary Talent (gary@drinkingtea.net). All rights reserved.
 */

#include <ox/std/defines.hpp>

#include <ox/claw/read.hpp>

#include "typeconv.hpp"

namespace nostalgia::core {

struct NostalgiaGraphicToTileSheetConverter: public Converter<NostalgiaGraphic, TileSheet> {
	ox::Error convert(NostalgiaGraphic *src, TileSheet *dst) noexcept final {
		dst->bpp              = src->bpp;
		dst->subsheet.name    = "Root";
		dst->subsheet.rows    = src->rows;
		dst->subsheet.columns = src->columns;
		dst->subsheet.size    = src->pixels.size();
		dst->defaultPalette   = std::move(src->defaultPalette);
		dst->pixels           = std::move(src->pixels);
		return OxError(0);
	}
};

struct NostalgiaPaletteToPaletteConverter: public Converter<NostalgiaPalette, Palette> {
	ox::Error convert(NostalgiaPalette *src, Palette *dst) noexcept final {
		dst->colors = std::move(src->colors);
		return OxError(0);
	}
};

template<typename Src>
struct ToCompactTileSheetConverter: public Converter<Src, CompactTileSheet> {
	ox::Error convert(Src *src, CompactTileSheet *dst) noexcept final {
		dst->bpp            = src->bpp;
		dst->defaultPalette = std::move(src->defaultPalette);
		dst->pixels         = std::move(src->pixels);
		return OxError(0);
	}
};

#ifndef OX_BARE_METAL
static const auto converters = [] {
	ox::Vector<ox::UniquePtr<BaseConverter>, 4> converters;
	converters.emplace_back(new NostalgiaGraphicToTileSheetConverter());
	converters.emplace_back(new NostalgiaPaletteToPaletteConverter());
	converters.emplace_back(new ToCompactTileSheetConverter<NostalgiaGraphic>());
	converters.emplace_back(new ToCompactTileSheetConverter<TileSheet>());
	return converters;
}();

static auto findConverter(const ox::String &srcTypeName, int srcTypeVersion, const ox::String &dstTypeName, int dstTypeVersion) noexcept -> ox::Result<BaseConverter*> {
	for (auto &c : converters) {
		if (c->matches(srcTypeName, srcTypeVersion, dstTypeName, dstTypeVersion)) {
			return c.get();
		}
	}
	return OxError(1, "Could not find converter");
};

ox::Error convert(const ox::Buffer &srcBuffer, const ox::String &dstTypeName, int dstTypeVersion, void *dstObj) noexcept {
	oxRequire(hdr, ox::readClawHeader(srcBuffer));
	oxRequire(c, findConverter(hdr.typeName, hdr.typeVersion, dstTypeName, dstTypeVersion));
	return c->convertRaw(srcBuffer, dstObj);
}
#endif

}
